/************************************************************************/
// Beispiel Ethernet Schnittstelle fr PIC18F9J60
// Programmierer: Fa. Schwabenplan, Ingo Schick
// Compiler: XC8
// Datum: 29.02.2020
/************************************************************************/

/************************************************************************/
// Include-Dateien
/************************************************************************/
#include <xc.h>                 // Standard-Bibiotheken einbeziehen
#include <stdio.h>              
#include <string.h>			
#include <stdint.h>
#include <stdbool.h>

//Display und IC-Bausteine
#include "mcc_generated_files/Display.H"
#include "mcc_generated_files/DS1337.H"
#include "mcc_generated_files/LM75.H"
#include "mcc_generated_files/24AA025E48.H"

//Config-Dateien fr IP- und MAC Adresse
#include "mcc_generated_files/TCPIPLibrary/ip_database.h"
#include "mcc_generated_files/TCPIPLibrary/mac_address.h"

//TCPIP Include-Dateien
#include "mcc_generated_files/mcc.h"
#include "mcc_generated_files/TCPIPLibrary/device_config.h"
#include "tcp_client_demo.h"
/************************************************************************/

/************************************************************************/
// sonstige Deklarationen
/************************************************************************/
#define True    1
#define False   0

#define ON      1
#define OFF     0
/************************************************************************/

/************************************************************************/
//Globale Variablen
/************************************************************************/
char chr_Temp[40];
date_time_t dt;
char chr_Wochentag[10];
int int_Temp;
/************************************************************************/

/************************************************************************/
// Hauptprogramm
/************************************************************************/
void main(void)
{

    SYSTEM_Initialize();
    
    TCP_Client_Initialize(Server_IP_a, Server_IP_b, Server_IP_c, Server_IP_d, usi_Port);
    
    LcdInit();
	LcdClear();
    
    LcdPrintf(0,0,"Ingenieurb\xf5ro Schwabenplan");
    LcdPrintf(1,24,"Temp:");
    LcdPrintf(1,34,"\xDF\C");
    LcdPrintf(2,0, "IP-Adresse:   ");
    LcdPrintf(3,0, "Subnet Maske: ");
    
    USART_Write_String("\n\r");
    USART_Write_String("Ingenieurbuero Schwabenplan\n\r\n\r");
    USART_Write_String("System Start PIC18F97J60\n\r\n\r");
    USART_Write_String("HW Version:     ");
    USART_Write_String(HARDWARE_VERSION "\n\r");
    USART_Write_String("Frequenz:       ");
    sprintf(chr_Temp, "%02u", usi_OSC);
    USART_Write_String(chr_Temp);
    USART_Write_String(OSC "\n\r\n\r");
    
    USART_Write_String("Hex-File:       ");
    USART_Write_String(HEXFILE "\n\r");
    USART_Write_String("SW Version:     ");
    USART_Write_String(SOFTWARE_VERSION "\n\r");
    USART_Write_String("Datum:          ");
    USART_Write_String(B_DATE "\n\r\n\r");
    
    USART_Write_String("USART-Port:     ");
    sprintf(chr_Temp, "%06lu", ulg_Baud);
    USART_Write_String(chr_Temp);
    USART_Write_String(" Baud \n\r");
    
    USART_Write_String("Interface:      ");
    sprintf(chr_Temp, "%03d", usi_I2C_Speed);
    USART_Write_String(chr_Temp);
    USART_Write_String(INTERFACE "\n\r\n\r");
    
    USART_Write_String("Bootloader:     ");
    USART_Write_String(BOOTLOADER_MODE "\n\r");
    
    USART_Write_String("Debug-Mode:     ");
    USART_Write_String(DEBUG_MODE "\n\r\n\r");
    
    uint32IP_to_String(ipdb_getAddress(), chr_Temp);
    USART_Write_String("IP-Adresse:     ");
    USART_Write_String(chr_Temp);
    LcdPrintf(2,15, chr_Temp);
    USART_Write_String("\n\r");
                
    uint32IP_to_String(ipdb_getSubNetMASK(), chr_Temp);
    USART_Write_String("Subnet Maske:   ");
    LcdPrintf(3,15, chr_Temp);
    USART_Write_String(chr_Temp);
    USART_Write_String("\n\r");

    uint32IP_to_String(ipdb_getRouter(), chr_Temp);
    USART_Write_String("Gateway:        ");
    USART_Write_String(chr_Temp);
    USART_Write_String("\n\r");
    
    USART_Write_String("MAC-Adresse:    ");
    MAC_to_String(chr_Temp);
    USART_Write_String(chr_Temp);
    USART_Write_String("\n\r\n\r");
    
    USART_Write_String("Server:         ");
    sprintf(chr_Temp, "%03lu.%03lu.%03lu.%03lu", Server_IP_a, Server_IP_b, Server_IP_c, Server_IP_d);
    USART_Write_String(chr_Temp);
    USART_Write_String("\n\r");
    
    USART_Write_String("Port:           ");
    sprintf(chr_Temp, "%05u", usi_Port);
    USART_Write_String(chr_Temp);
    USART_Write_String("\n\r\n\r");
    
    USART_Write_String("RTC-Typ:        ");
    USART_Write_String(RTC_TYP "\n\r");
    
    USART_Write_String("Temp-Sensor:    ");
    USART_Write_String(Temp_Sensor_TYP "\n\r");
    USART_Write_String("Sensor-Adresse: ");
    sprintf(chr_Temp, "%01d", usi_Temp_Sensor_Adr);
    USART_Write_String(chr_Temp);
    USART_Write_String("\n\r");
    
    dt.Stunden = DS1337_Read_Stunden(&dt);
    dt.Minuten = DS1337_Read_Minuten(&dt);
    dt.Sekunden = DS1337_Read_Sekunden(&dt);
		
    dt.Tag = DS1337_Read_Tag(&dt);
    dt.Monat = DS1337_Read_Monat(&dt);
    dt.Jahr = DS1337_Read_Jahr(&dt);
    dt.Wochentag = DS1337_Read_Wochentag(&dt);
    
    strcpy(chr_Wochentag, Wochentag_Namen[dt.Wochentag]); 
    sprintf(chr_Temp, "%s %02u.%02u.20%02u %02u:%02u:%02u ",chr_Wochentag, dt.Tag, dt.Monat, dt.Jahr, dt.Stunden, dt.Minuten, dt.Sekunden);
    LcdPrintf(1,0, chr_Temp);
    USART_Write_String("\n\r");
    USART_Write_String("Board RTC:      ");
    USART_Write_String(chr_Temp);
    USART_Write_String("\n\r");
    
    sprintf(chr_Temp, "%.4u",int_Temp = GetTempLM75(1));
    LcdPrintf(1,30, chr_Temp);
    USART_Write_String("Board Temp:     ");
    USART_Write_String(chr_Temp);
    USART_Write_String(" C\n\r");
    
    
    // If using interrupts in PIC18 High/Low Priority Mode you need to enable the Global High and Low Interrupts
    // If using interrupts in PIC Mid-Range Compatibility Mode you need to enable the Global and Peripheral Interrupts
    // Use the following macros to:

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();
    
    
    while (1)
    {
        Network_Manage();
        DEMO_TCP_Client();     
    }
}
