/************************************************************************/
// DS1337.C fr PIC18F97J60
// Programmierer: Fa. Schwabenplan, Ingo Schick
// Compiler: XC8
// Datum: 08.02.2020
/************************************************************************/

#ifndef __DS1337_H
#define __DS1337_H

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>

/************************************************************************/             
//Wochentag Namen
/************************************************************************/ 
char const Wochentag_Namen[8][4] =
{
{"   "},	
{"So."},
{"Mo."},
{"Di."},
{"Mi."},
{"Do."},
{"Fr."},
{"Sa."}
};   
/************************************************************************/

/************************************************************************/               
//Struktur Datum und Uhrzeit
/************************************************************************/ 
typedef struct
{
unsigned int Sekunden;    	// 0 to 59
unsigned int Minuten;    	// 0 to 59
unsigned int Stunden;		// 0 to 23  (24 Stunden-Modus)
unsigned int Wochentag;    	// 0 = Sonntag, 1 = Montag, etc.
unsigned int Tag;        	// 1 to 31
unsigned int Monat;      	// 1 to 12
unsigned int Jahr;       	// 00 to 99
}date_time_t;
/************************************************************************/

char bcd2bin(char bcd_value);
unsigned int bin2bcd(unsigned int value);

unsigned int DS1337_Write_Byte(unsigned int address, unsigned int data);
unsigned int DS1337_Read_Byte(unsigned int address);

void DS1337_Write_Stunden(date_time_t *dt);
void DS1337_Write_Minuten(date_time_t *dt);
void DS1337_Write_Sekunden(date_time_t *dt);
void DS1337_Write_Wochentag(date_time_t *dt);
void DS1337_Write_Tag(date_time_t *dt);
void DS1337_Write_Monat(date_time_t *dt);
void DS1337_Write_Jahr(date_time_t *dt);

unsigned int DS1337_Read_Stunden(date_time_t *dt);
unsigned int DS1337_Read_Minuten(date_time_t *dt);
unsigned int DS1337_Read_Sekunden(date_time_t *dt);
unsigned int DS1337_Read_Wochentag(date_time_t *dt);
unsigned int DS1337_Read_Tag(date_time_t *dt);
unsigned int DS1337_Read_Monat(date_time_t *dt);
unsigned int DS1337_Read_Jahr(date_time_t *dt);

void DS1337_Reset(void);

#endif
