/************************************************************************/
// 	Hauptprogramm LCD-Display 4x40
// 	Autor: Ingo Schick
//	Datum: 28.01.2010
/************************************************************************/

#include "Display.H"
#include <string.h>

// Prototypendefinition, Achtung nur fr interne Verwendung des Moduls
void LcdPutInitCmd(int Cmd, int UpLow);
void LcdEnable(int UpLow);
void LcdBusy(int UpLow);
void LcdHome(int UpLow);
void LcdEntryMode(int mode, int UpLow);
void LcdSetDisplayMode(int mode, int UpLow);
void LcdSetCharacterGeneratorAddress(int addr, int UpLow);
void LcdSetDisplayDataAddress(int addr, int UpLow);
int LcdGetAddress(int UpLow);
void LcdPutChar(char Value, int UpLow);
void LcdPutCmd(int Cmd, int UpLow);



void LcdPutCString(int Position, char* pString, int UpLow);

#define bit_set(a,b) a.b = 1;
#define bit_clear(a,b) a.b = 0;


char cString[40];


unsigned long wData;
unsigned long wDisplayInitFlag;
BYTE bCurrentLine, bCurrentPos;

/************************************************************************/
// LcdInit
// LC-Display Initalisierung
/************************************************************************/
void LcdInit(void)
{
  UBYTE i;
  wData = 2010;
 /************************************************************************/
	  // 1. LCD-Init
	  LcdPutInitCmd(0x30, UPPER);
      __delay_ms(4);
	  LcdPutInitCmd(0x30, LOWER);
      __delay_ms(4);
	  LcdPutInitCmd(0x20, UPPER);
      __delay_ms(4);
	  LcdPutInitCmd(0x20, LOWER);
	  __delay_ms(4);
	  LcdPutCmd(0x28, UPPER);
      __delay_ms(4);
	  LcdPutCmd(0x28, LOWER);
	  __delay_ms(4);
	  
	  LcdSetDisplayMode(0x04, UPPER);
      __delay_ms(4);
	  LcdSetDisplayMode(0x04, LOWER);
      __delay_ms(4);
	  LcdClear();
	  
	  LcdEntryMode(0x06, UPPER);
      __delay_ms(4);
	  LcdEntryMode(0x06, LOWER);
      __delay_ms(4);
	  wDisplayInitFlag = 0xa5b6;	
/************************************************************************/	  
	  // 2. LCD-Init
	  LcdPutInitCmd(0x30, UPPER);
      __delay_ms(4);
	  LcdPutInitCmd(0x30, LOWER);
      __delay_ms(4);
	  LcdPutInitCmd(0x20, UPPER);
      __delay_ms(4);
	  LcdPutInitCmd(0x20, LOWER);
      __delay_ms(4);
	  LcdPutCmd(0x28, UPPER);
      __delay_ms(4);
	  LcdPutCmd(0x28, LOWER);
	  __delay_ms(4);
	  
	  LcdSetDisplayMode(0x04, UPPER);
      __delay_ms(4);
	  LcdSetDisplayMode(0x04, LOWER);
      __delay_ms(4);
	  LcdClear();
	  
	  LcdEntryMode(0x06, UPPER);
      __delay_ms(4);
	  LcdEntryMode(0x06, LOWER);
      __delay_ms(4);
      wDisplayInitFlag = 0xa5b6;
	

  bCurrentLine = INVALID;
  bCurrentPos = INVALID;
}
/************************************************************************/


/************************************************************************/
// LcdEnable
// setzt EN1 oder EN 2 des LC-Displays auf High-Pegel
/************************************************************************/
void LcdEnable(int UpLow)
{
	if (UpLow == UPPER)
	     LCD_CTRL.RE2 = 1;
	  else 
 		 LCD_CTRL.RE3 = 1; 
}
/************************************************************************/

/************************************************************************/
// LcdDisable
// setzt EN1 oder EN 2 des LC-Displays auf Low-Pegel
/************************************************************************/
void LcdDisable(int UpLow)
{
	if (UpLow == UPPER)
	     LCD_CTRL.RE2 = 0;
	  else 
 		 LCD_CTRL.RE3 = 0;
}
/************************************************************************/


/************************************************************************/
// LcdBusy
// Returns when LCD busy-flag is inactive
/************************************************************************/
void LcdBusy(int UpLow)
{
  UBYTE tmp, tmp2;
  LCD_DATA_TRIS |= 0xf0; // PORTD auf Empfang umschalten
  tmp = 0;
  bit_clear(LCD_CTRL, LCD_RS);
  bit_set(LCD_CTRL, LCD_RW);
  do
    {
      
	  LcdEnable(UpLow);
      tmp = 0;
      tmp = 0;
      tmp = LCD_DATA;		// Read busy flag + DDram address
      LcdDisable(UpLow);	// LCD E-line Low
	  tmp2 = 0;
      tmp2 = 0;
      LcdEnable(UpLow);
      tmp2 = 0;
      tmp2 = 0;
      tmp2 = LCD_DATA;		// Read busy flag + DDram address
      LcdDisable(UpLow);	// LCD E-line Low
      tmp = (tmp&0xf0) | ((tmp2&0xf0)>>4);
    }
  while((tmp&(1<<7)) != 0);
  bit_clear(LCD_CTRL, LCD_RW);
  LCD_DATA_TRIS = 0x00;		// PORTD auf Ausgabe umschalten
}
/************************************************************************/


/************************************************************************/
// LcdHome
// Cursor kehrt an Zeilenanfang zurck
// UpLow Werte 0 fr oberes LC-Display und 1 fr unteres LC-Display
/************************************************************************/
void LcdHome(int UpLow)
{
  LcdPutCmd(0x02, UpLow);
}
/************************************************************************/


/************************************************************************/
// LcdEntryMode
// setzt den Eingabemodus des LC-Display
// Der Eingabemodus mu  definiert werden
//  b0	: 0 = no display shift	1 = display shift
//  b1	: 0 = auto-decrement	1 = auto-increment
//  b2-7	: don't care
// UpLow Werte 0 fr oberes LC-Display und 1 fr unteres LC-Display
/************************************************************************/
void LcdEntryMode(int mode, int UpLow)
{
  LcdPutCmd((mode&0x03)|0x04, UpLow);
}
/************************************************************************/

/************************************************************************/
// LcdSetDisplayMode
// setzt die Controlle des LC-Display
// LC-Diplay mode wird in W definiert
//  b0	: 0 = Cursor blinken off	1 = Cursor blinken on
//  b1	: 0 = Cursor off	1 = Cursor on
//  b2	: 0 = Display off	1 = Display on (Display data remains in DDRAM)
//  b3-7	: don't care
// UpLow Werte 0 fr oberes LC-Display und 1 fr unteres LC-Display
/************************************************************************/
void LcdSetDisplayMode(int mode, int UpLow)
{
	LcdPutCmd((mode&0x07)|0x08, UpLow);
}
/************************************************************************/


/************************************************************************/
// LcdSetCharacterGeneratorAddress
/************************************************************************/
void LcdSetMode(int mode)
{
	LcdSetDisplayMode(mode, UPPER);
	LcdSetDisplayMode(mode, LOWER);
}
/************************************************************************/


/************************************************************************/
// LcdSetCharacterGeneratorAddress
// Sets Character-Generator-RAM address. CGRAM is read/written after
//  this setting.
// Required CGRAM address must be set in addr
//  b0-5	: required CGRAM address
//  b6-7	: don't care
/************************************************************************/
void LcdSetCharacterGeneratorAddress(int addr, int UpLow)
{
  LcdPutCmd((addr&0x3f)|0x40, UpLow);
}
/************************************************************************/

/************************************************************************/
// LcdSetDisplayDataAddress
// Sets the Display-Data-RAM address. DDRAM data is read/written after
//  this setting.
// Required DDRAM address must be set in addr
//  b0-6	: required DDRAM address
//  b7	: don't care
/************************************************************************/
void LcdSetDisplayDataAddress(int addr, int UpLow)
{
  LcdPutCmd(addr|0x80, UpLow);
}
/************************************************************************/


/************************************************************************/
// LCDGetAddress
// Returns address counter contents, used for both DDRAM and CGRAM.
// RAM address is returned
/************************************************************************/

int LcdGetAddress(int UpLow)
{
  UBYTE tmp, tmp2;

  LCD_DATA_TRIS = 0xff; // PORTD auf Empfang umschalten
  tmp = 0;
  bit_clear(LCD_CTRL, LCD_RS);
  bit_set(LCD_CTRL, LCD_RW);
  
  LcdEnable(UpLow);
  tmp = LCD_DATA;		// Read busy flag + DDram address; bits 7..4
  LcdDisable(UpLow);	// LCD E-line Low
  
  LcdEnable(UpLow);
  tmp2 = LCD_DATA;		// Read busy flag + DDram address; bits 3..0
  LcdDisable(UpLow);	// LCD E-line Low
  
  bit_clear(LCD_CTRL, LCD_RW);
  LCD_DATA_TRIS = 0x00;	// PORTD auf Ausgabe umschalten
  return (tmp<<4) | (tmp2&0x0f);
  
}
/************************************************************************/


/************************************************************************/
// LcdPutChar
// Sends Character to LCD
// Required Character must be in Value
/************************************************************************/

void LcdPutChar (char Value, int UpLow)
{
  LcdBusy(UpLow);
  bit_clear(LCD_CTRL, LCD_RW);    	// Set LCD in write mode
  bit_set(LCD_CTRL, LCD_RS);
  #ifdef _EIGHT_BIT_MODE
    bit_set(LCD_CTRL, UpLow);   	// Set LCD in data mode and LCD E-line High
    LCD_DATA = Value;	        	// Send data to LCD
    bit_clear(LCD_CTRL, UpLow);		// LCD E-line Low
  #else
    LcdEnable(UpLow);       		// Set LCD E-line High
	LCD_DATA = (1<<UpLow) | (0x01) | (Value&0xf0);	// Send data to LCD
	LcdDisable(UpLow);  			// LCD E-line Low
	LcdEnable(UpLow);       		// Set LCD E-line High
	LCD_DATA = (1<<UpLow)|(0x01)|(Value<<4);	// Send data to LCD
    LcdDisable(UpLow);	    		// LCD E-line Low
  #endif
  bit_clear(LCD_CTRL, LCD_RS);
}
/************************************************************************/


/************************************************************************/
// LcdPutCString
// Writes a C compatible string to the display
// uses LcdPutChar
/************************************************************************/
void LcdPutCString(int Position, char* pString, int UpLow)
{

  LcdSetDisplayDataAddress(Position, UpLow);

  while (*pString != '\0')
    {
      LcdPutChar(*pString, UpLow);
      pString++;
    }

}
/************************************************************************/


/************************************************************************/
// LCDPutInitCmd
// Sends command to LCD during init -- in 8 bit mode still
/************************************************************************/
void LcdPutInitCmd(int Cmd, int UpLow)
{
	bit_clear(LCD_CTRL, LCD_RW);
    bit_clear(LCD_CTRL, LCD_RS);    // Set LCD in write mode and Set LCD in command mode
	LcdEnable(UpLow);       // Set LCD E-line High
	LCD_DATA = (1<<UpLow) | (Cmd);	// Keep LCD_E high and send data to LCD 
	LcdDisable(UpLow);	    // LCD E-line low (takes over data in LCD)
}
/************************************************************************/


/************************************************************************/
// LCDPutCmd
// Sends command to LCD using 4 bit commands
// display must be in 4 bit mode in order to understand th protocol
/************************************************************************/
void LcdPutCmd(int Cmd, int UpLow)
{
	LcdBusy(UpLow);
	bit_clear(LCD_CTRL, LCD_RW);
    bit_clear(LCD_CTRL, LCD_RS);    // Set LCD in write mode and Set LCD in command mode
	LcdEnable(UpLow);       // Set LCD E-line High
	LCD_DATA = (1<<UpLow)|(Cmd&0xf0);	// Send data to LCD
	LcdDisable(UpLow);		// LCD E-line Low
	LcdEnable(UpLow);       // Set LCD E-line High
	LCD_DATA = (1<<UpLow)|(Cmd<<4);	// Send data to LCD
    LcdDisable(UpLow);	    // LCD E-line Low
}
/************************************************************************/


/************************************************************************/
// LcdPutCharacter -- public function
// writes character to LCD at position given by position and line, 
// Required Character must be in Value
/************************************************************************/
void LcdPutCharacter(int Line, int Position, char Value)
{
	BYTE UpLow;

	switch (Line)
	{
	case 0:
	     LcdSetDisplayDataAddress(Position, UPPER);
		 UpLow = UPPER;
	     break;
	case 1:
	     LcdSetDisplayDataAddress(0x40+Position, UPPER);
		 UpLow = UPPER;
	     break;
	case 2:
	     LcdSetDisplayDataAddress(Position, LOWER);
		 UpLow = LOWER;
	     break;
	case 3:
	     LcdSetDisplayDataAddress(0x40+Position, LOWER);
		 UpLow = LOWER;
	     break;
	default: ;
	}  

	LcdPutChar (Value, UpLow);
	bCurrentLine = INVALID;
    bCurrentPos = INVALID;
}
/************************************************************************/


/************************************************************************/
// LcdPutCharacterHere -- public function
// writes character to LCD at current position set before in LcdSetCursorPosition
// Required Character must be in Value
/************************************************************************/
int LcdPutCharacterHere(char Value)
{


	switch (bCurrentLine)
	{
	case 0:
	case 1:
	     LcdPutChar (Value, UPPER);
		 bCurrentPos++;
		 return 0;
	     break;
	case 2:
	case 3:
	     LcdPutChar (Value, LOWER);
		 bCurrentPos++;
		 return 0;
	     break;
	default: ;
	}  
	return -1;
}

int LcdGetCurrentLine(void)
{
	return bCurrentLine;
}

int LcdGetCurrentPosition(void)
{
	return bCurrentPos;
}
/************************************************************************/


/************************************************************************/
// LcdPrintf
// Writes a C compatible string to the display
// formats it by using library internal vsprintf function
// uses LcdPrintf
/************************************************************************/
void LcdPrintf(int Line, int Position, char *fmt)
{

    sprintf(cString, fmt);
	switch(Line)
	{
	case 0:
		LcdPutCString(Position, cString, UPPER);
		break;
	case 1:
		LcdPutCString(0x40+Position, cString, UPPER);
		break;
	case 2:
		LcdPutCString(Position, cString, LOWER);
		break;
	case 3:
		LcdPutCString(0x40+Position, cString, LOWER);
		break;
	default: ;
	}
	bCurrentLine = INVALID;
    bCurrentPos = INVALID;
}
/************************************************************************/


/************************************************************************/
// LcdSetCursorPosition 
// Sends cursor position to LCD
// Required values are line 0..3, position 0..26, CursorMode see above
/************************************************************************/
void LcdSetCursorPosition(int Line, int Position, int CursorMode)
{

	switch (Line)
	{
	case 0:
	     LcdSetDisplayDataAddress(Position, UPPER);
		 if (CursorMode != UNCHANGED)
			 LcdSetDisplayMode(CursorMode, UPPER);
		 LcdSetDisplayMode(DISPLAY_ON, LOWER);
	     break;
	case 1:
	     LcdSetDisplayDataAddress(0x40+Position, UPPER);
		 if (CursorMode != UNCHANGED)
			 LcdSetDisplayMode(CursorMode, UPPER);
		 LcdSetDisplayMode(DISPLAY_ON, LOWER);
	     break;
	case 2:
	     LcdSetDisplayDataAddress(Position, LOWER);
		 if (CursorMode != UNCHANGED)
			 LcdSetDisplayMode(CursorMode, LOWER);
		 LcdSetDisplayMode(DISPLAY_ON, UPPER);
	     break;
	case 3:
	     LcdSetDisplayDataAddress(0x40+Position, LOWER);
		 if (CursorMode != UNCHANGED)
			 LcdSetDisplayMode(CursorMode, LOWER);
		 LcdSetDisplayMode(DISPLAY_ON, UPPER);
	     break;
	default: ;
	}  
	bCurrentLine = Line;
	bCurrentPos = Position;
}
/************************************************************************/


/************************************************************************/
// LCDCLEAR
// Clears display and returns cursor to home position (upper-left corner).
/************************************************************************/
void LcdClear(void)
{
	 LcdPutCmd(0x01, UPPER);
	 LcdPutCmd(0x01, LOWER);
	 bCurrentLine = INVALID;
	 bCurrentPos = INVALID;
}
/************************************************************************/


/************************************************************************/
// LcdClearLine
// clears one single line of the display
// resets also its flashing properties
// you must give the logical line number in parameter Line, not the physical address
/************************************************************************/
void LcdClearLine(UBYTE Line)
{
  BYTE i = 0;
  BYTE UpLow;
  switch (Line)
  {
	case 0:
	     LcdSetDisplayDataAddress(0, UPPER);
		 UpLow = UPPER;
	     break;
	case 1:
	     LcdSetDisplayDataAddress(0x40, UPPER);
		 UpLow = UPPER;
	     break;
	case 2:
	     LcdSetDisplayDataAddress(0, LOWER);
		 UpLow = LOWER;
	     break;
	case 3:
	     LcdSetDisplayDataAddress(0x40, LOWER);
		 UpLow = LOWER;
	     break;
	default: ;
     
  }  
  for (i=0; i<LINELENGTH; i++)
  {
	LcdPutChar(' ', UpLow);
  }
  bCurrentLine = INVALID;
  bCurrentPos = INVALID;
}
/************************************************************************/


