/**
  Descriptive File Name
	
  Company:
    Microchip Technology Inc.

  File Name:
    log_syslog.h

  Summary:
    This is the header file for the log_syslog.c

  Description:
    This header file provides the API for sending log messages using Ethernet (Syslog RFC 5424).

 */

/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef LOG_ETHERNET_H
#define	LOG_ETHERNET_H

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h>
#include "tcpip_types.h"

#define SOURCEPORT_SYSLOG   514
#define DESTPORT_SYSLOG     514


/*Send a syslog message.
 * The function will send a Syslog message.
 * 
 * @param message
 *      Message
 * 
 * @param priorityVal
 *      Message priority
 * 
 * @param return
 *      Status of sending the message
 * 
 */
error_msg logSyslog(const char *message, uint8_t priorityVal);

#endif	/* LOG_ETHERNET_H */

