/************************************************************************/
// ADW_PIC18F97K60.C fr PIC18F97J60
// Programmierer: Fa. Schwabenplan, Ingo Schick
// Compiler: XC8
// Datum: 25.08.2018
/************************************************************************/

#include <xc.h>
#include "adc.h"
#include "TCPIPLibrary/device_config.h"

void (*ADC_InterruptHandler)(void);

void ADC_Initialize(void)
{
    // set the ADC to the options selected in the User Interface
    
    // GO_nDONE stop; ADCAL Normal_a/d_operation; ADON enabled; CHS AN2; 
    ADCON0 = 0x09;
    
    // VCFG1 VSS; VCFG0 VDD; 
    ADCON1 = 0x00;
    
    // ADFM right; ACQT 8; ADCS FOSC/32; 
    ADCON2 = 0xA2;
    
    // ADRESL 0; 
    ADRESL = 0x00;
    
    // ADRESH 0; 
    ADRESH = 0x00;
    
}

void ADC_SelectChannel(adc_channel_t channel)
{
    // select the A/D channel
    ADCON0bits.CHS = channel;    
    // Turn on the ADC module
    ADCON0bits.ADON = 1;  
}

void ADC_StartConversion()
{
    // Start the conversion
    ADCON0bits.GO_nDONE = 1;
}


bool ADC_IsConversionDone()
{
    // Start the conversion
   return ((bool)(!ADCON0bits.GO_nDONE));
}

adc_result_t ADC_GetConversionResult(void)
{
    // Conversion finished, return the result
    return ((adc_result_t)((ADRESH << 8) + ADRESL));
}

adc_result_t ADC_GetConversion(adc_channel_t channel)
{
    // select the A/D channel
    ADCON0bits.CHS = channel;    

    // Turn on the ADC module
    ADCON0bits.ADON = 1;

    // Start the conversion
    ADCON0bits.GO_nDONE = 1;

    // Wait for the conversion to finish
    while (ADCON0bits.GO_nDONE)
    {
    }

    // Conversion finished, return the result
    return ((adc_result_t)((ADRESH << 8) + ADRESL));
}

void ADC_TemperatureAcquisitionDelay(void)
{
    __delay_us(200);
}
