/************************************************************************/
// System-Init fr PIC18F9J60
// Programmierer: Fa. Schwabenplan, Ingo Schick
// Compiler: XC8
// Datum: 29.02.2020
/************************************************************************/

#include "mcc.h"

void SYSTEM_Initialize(void)
{


    INTERRUPT_Initialize();
    OSCILLATOR_Initialize();
    PIN_MANAGER_Initialize();
    
    ADC_Initialize();
    SPI1_Initialize();
    TMR1_Initialize();
   
    // Init_I2C2(usi_I2C_Speed);
    Init_I2C2(100);
    
    // I2C-Gewschwindigkeit auslesen
    // mit 100 KHz starten -> Geschwindigkeit auslesen und schlieen
    usi_I2C_Speed = ((Read_Byte_24AA025E48(0, 0x2A)) * 100);
    CloseI2C2();
    
    // I2C mit gespeicherter Geschwindigkeit wieder starten
    Init_I2C2(usi_I2C_Speed);
    
    
    // MAC-Adresse auslesen
    chr_MAC[0] = Read_Byte_24AA025E48(0, 0xFA);
    chr_MAC[1] = Read_Byte_24AA025E48(0, 0xFB);
    chr_MAC[2] = Read_Byte_24AA025E48(0, 0xFC);
    chr_MAC[3] = Read_Byte_24AA025E48(0, 0xFD);
    chr_MAC[4] = Read_Byte_24AA025E48(0, 0xFE);
    chr_MAC[5] = Read_Byte_24AA025E48(0, 0xFF);
    
    
    // IP-Adresse auslesen
    chr_IP[0] = Read_Byte_24AA025E48(0, 0);
    chr_IP[1] = Read_Byte_24AA025E48(0, 1);
    chr_IP[2] = Read_Byte_24AA025E48(0, 2);
    chr_IP[3] = Read_Byte_24AA025E48(0, 3);
    
    ipv4_IP =  (uint32_t) chr_IP[0]<<24 | (uint32_t) chr_IP[1]<<16 | (uint32_t) chr_IP[2]<<8 | (uint32_t) chr_IP[3];
    
    // Subnet-Maske auslesen
    chr_SN[0] = Read_Byte_24AA025E48(0, 4);
    chr_SN[1] = Read_Byte_24AA025E48(0, 5);
    chr_SN[2] = Read_Byte_24AA025E48(0, 6);
    chr_SN[3] = Read_Byte_24AA025E48(0, 7);
    
    ipv4_subnetMask  =  (uint32_t) chr_SN[0]<<24 | (uint32_t) chr_SN[1]<<16 | (uint32_t) chr_SN[2]<<8 | (uint32_t) chr_SN[3];
    
    // Router auslesen
    chr_ROU[0] = Read_Byte_24AA025E48(0, 8);
    chr_ROU[1] = Read_Byte_24AA025E48(0, 9);
    chr_ROU[2] = Read_Byte_24AA025E48(0, 10);
    chr_ROU[3] = Read_Byte_24AA025E48(0, 11);
    
    ipv4_router  =  (uint32_t) chr_ROU[0]<<24 | (uint32_t) chr_ROU[1]<<16 | (uint32_t) chr_ROU[2]<<8 | (uint32_t) chr_ROU[3];
 
    // Gateway auslesen
    chr_GW[0] = Read_Byte_24AA025E48(0, 12);
    chr_GW[1] = Read_Byte_24AA025E48(0, 13);
    chr_GW[2] = Read_Byte_24AA025E48(0, 14);
    chr_GW[3] = Read_Byte_24AA025E48(0, 15);
    
    ipv4_gateway  =  (uint32_t) chr_GW[0]<<24 | (uint32_t) chr_GW[1]<<16 | (uint32_t) chr_GW[2]<<8 | (uint32_t) chr_GW[3];  
    
    // DNS auslesen
    chr_DNS[0] = Read_Byte_24AA025E48(0, 16);
    chr_DNS[1] = Read_Byte_24AA025E48(0, 17);
    chr_DNS[2] = Read_Byte_24AA025E48(0, 18);
    chr_DNS[3] = Read_Byte_24AA025E48(0, 19);
    
    ipv4_dns  =  (uint32_t) chr_DNS[0]<<24 | (uint32_t) chr_DNS[1]<<16 | (uint32_t) chr_DNS[2]<<8 | (uint32_t) chr_DNS[3];  
    
    // IP-Adresse des Server auslesen
    Server_IP_a = Read_Byte_24AA025E48(0, 20);                        
    Server_IP_b = Read_Byte_24AA025E48(0, 21); 
    Server_IP_c = Read_Byte_24AA025E48(0, 22); 
    Server_IP_d = Read_Byte_24AA025E48(0, 23); 
    
    // Port des Server auslesen
    chr_Port[0] = Read_Byte_24AA025E48(0, 24);
    chr_Port[1] = Read_Byte_24AA025E48(0, 25);
    
    //uint_Port =  0xFF00;
    usi_Port =  ((chr_Port[0] << 8)) + chr_Port[1];

 
    // Baudrate USART auslesen
    chr_USART = Read_Byte_24AA025E48(0, 0x29);
    switch(chr_USART)
    {
        case 1:  ulg_Baud = 9600; break;
        case 2:  ulg_Baud = 19200; break;
        case 3:  ulg_Baud = 57600; break;
        case 4:  ulg_Baud = 115200; break;

        default: ulg_Baud = 115200;;
    }
    
    
    
    Init_USART(ulg_Baud);
    
    Network_Init();
}

void OSCILLATOR_Initialize(void)
{
    // SCS Primary_OSC; OSTS intosc; IDLEN disabled; 
    OSCCON = 0x02;
    // PLLEN disabled; PPST0 disabled; PPRE divide_by_3; PPST1 divide_by_3; 
    OSCTUNE = 0x00;
}
