/************************************************************************/
// Timer 1 fr PIC18F9J60
// Programmierer: Fa. Schwabenplan, Ingo Schick
// Compiler: XC8
// Datum: 29.02.2020
/************************************************************************/

/************************************************************************/
// Include-Dateien
/************************************************************************/
#include <xc.h>
#include "tmr1.h"

/************************************************************************/
// globale Variablen
/************************************************************************/
char chr_Temp[40];
date_time_t dt;
char chr_Wochentag[10];
int int_Temp;
/************************************************************************/

volatile uint16_t timer1ReloadVal;
void (*TMR1_InterruptHandler)(void);

/**
  Section: TMR1 APIs
*/

void TMR1_Initialize(void)
{
    //Set the Timer to the options selected in the GUI
	
	// TMR1H 128; 
		TMR1H = 0x80;
	
	// TMR1L 0; 
		TMR1L = 0x00;

    // Load the TMR value to reload variable
    timer1ReloadVal=TMR1;

    // Clearing IF flag before enabling the interrupt.
    PIR1bits.TMR1IF = 0;

    // Enabling TMR1 interrupt.
    PIE1bits.TMR1IE = 1;

    // Set Default Interrupt Handler
    TMR1_SetInterruptHandler(TMR1_DefaultInterruptHandler);

    // T1CKPS 1:1; T1OSCEN enabled; TMR1CS External; nT1SYNC synchronize; TMR1ON enabled; RD16 disabled; 
    T1CON = 0x0B;
}

void TMR1_StartTimer(void)
{
    // Start the Timer by writing to TMRxON bit
    T1CONbits.TMR1ON = 1;
}

void TMR1_StopTimer(void)
{
    // Stop the Timer by writing to TMRxON bit
    T1CONbits.TMR1ON = 0;
}

uint16_t TMR1_ReadTimer(void)
{
    uint16_t readVal;
    uint8_t readValHigh;
    uint8_t readValLow;
	
    T1CONbits.RD16 = 1;
	
    readValLow = TMR1L;
    readValHigh = TMR1H;
    
    readVal = ((uint16_t)readValHigh << 8) | readValLow;

    return readVal;
}

void TMR1_WriteTimer(uint16_t timerVal)
{
    if (T1CONbits.nT1SYNC == 1)
    {
        // Stop the Timer by writing to TMRxON bit
        T1CONbits.TMR1ON = 0;

        // Write to the Timer1 register
        TMR1H = (timerVal >> 8);
        TMR1L = (uint8_t) timerVal;

        // Start the Timer after writing to the register
        T1CONbits.TMR1ON =1;
    }
    else
    {
        // Write to the Timer1 register
        TMR1H = (timerVal >> 8);
        TMR1L = (uint8_t) timerVal;
    }
}

void TMR1_Reload(void)
{
    TMR1_WriteTimer(timer1ReloadVal);
}

void TMR1_ISR(void)
{
    // Clear the TMR1 interrupt flag
    PIR1bits.TMR1IF = 0;    
    TMR1_WriteTimer(timer1ReloadVal);
    
    dt.Stunden = DS1337_Read_Stunden(&dt);
    dt.Minuten = DS1337_Read_Minuten(&dt);
    dt.Sekunden = DS1337_Read_Sekunden(&dt);
		
    dt.Tag = DS1337_Read_Tag(&dt);
    dt.Monat = DS1337_Read_Monat(&dt);
    dt.Jahr = DS1337_Read_Jahr(&dt);
    dt.Wochentag = DS1337_Read_Wochentag(&dt);
				
    strcpy(chr_Wochentag, Wochentag_Namen[dt.Wochentag]); 
    sprintf(chr_Temp, "%s %02u.%02u.20%02u %02u:%02u:%02u ",chr_Wochentag, dt.Tag, dt.Monat, dt.Jahr, dt.Stunden, dt.Minuten, dt.Sekunden);
    LcdPrintf(1,0, chr_Temp);
    USART_Write_String("\n\r");
    USART_Write_String(chr_Temp);
    USART_Write_String("\n\r");

    sprintf(chr_Temp, "%.4u",int_Temp = GetTempLM75(usi_Temp_Sensor_Adr));
    LcdPrintf(1,30, chr_Temp); 
    USART_Write_String("Board Temp: ");
    USART_Write_String(chr_Temp);
    USART_Write_String(" C\n\r\n\r");

    // ticker function call;
    // ticker is 1 -> Callback function gets called every time this ISR executes
    TMR1_CallBack();
}

void TMR1_CallBack(void)
{
    // Add your custom callback code here

    if(TMR1_InterruptHandler)
    {
        TMR1_InterruptHandler();
    }
}

void TMR1_SetInterruptHandler(void (* InterruptHandler)(void)){
    TMR1_InterruptHandler = InterruptHandler;
}

void TMR1_DefaultInterruptHandler(void){
    // add your TMR1 interrupt custom code
    // or set custom function using TMR1_SetInterruptHandler()
}
