/************************************************************************/
// TCP-Client fr PIC18F9J60
// Programmierer: Fa. Schwabenplan, Ingo Schick
// Compiler: XC8
// Datum: 29.02.2020
/************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "mcc_generated_files/mcc.h"
#include "tcp_client_demo.h"
#include "mcc_generated_files/TCPIPLibrary/tcpv4.h"
#include "mcc_generated_files/TCPIPLibrary/ipv4.h"
#include "mcc_generated_files/TCPIPLibrary/tcpip_config.h"

sockaddr_in4_t remoteSocket;

void DEMO_TCP_Client(void)
{
    // create the socket for the TCP Client
    static tcpTCB_t port60TCB;
    
    // create the TX and RX Client's buffers
    static uint8_t rxdataPort60[50];
    static uint8_t txdataPort60[80];


    static time_t t_client;
    static time_t socketTimeout;
    
    uint16_t rx_len;
    socketState_t socketState;
    rx_len = 0;
    uint16_t pot;
    char strTmp[17];
    

    socketState = TCP_SocketPoll(&port60TCB);

    time(&t_client);

    switch(socketState)
    {
        case NOT_A_SOCKET:
            // Inserting and initializing the socket
            TCP_SocketInit(&port60TCB);
            break;
        case SOCKET_CLOSED:
            // if the socket is closed we will try to connect again
            // try to connect once at 2 seconds
            socketTimeout = t_client + 2;
            TCP_InsertRxBuffer(&port60TCB, rxdataPort60, sizeof(rxdataPort60));
            TCP_Connect(&port60TCB, &remoteSocket);
            break;
        case SOCKET_IN_PROGRESS:
            // if the socket is closed we will try to connect again
            if(t_client >= socketTimeout)
            {
                TCP_Close(&port60TCB);
            }
            break;
        case SOCKET_CONNECTED:
            // implement an echo client over TCP
            // check if the previous buffer was sent
            if (TCP_SendDone(&port60TCB))
            {
                rx_len = TCP_GetReceivedData(&port60TCB);
                // handle the incoming data
                if(rx_len > 0)
                {
                    // check for "led x on/off" command
                    if (rx_len > 16) 
                    {
                        rxdataPort60[16] = 0;
                    } else 
                    {
                        rxdataPort60[rx_len] = 0;
                    }

                    if(rxdataPort60[0] == 'l' && rxdataPort60[1] == 'e' && rxdataPort60[2] == 'd') 
                    {
                        if(rxdataPort60[6] == 'o' && rxdataPort60[7] == 'n') 
                        {
                            switch(rxdataPort60[4])
                            {
                                case '0':        
                                    RJ0_SetHigh();
                                    break;
                                case '1':       
                                    RJ1_SetHigh();
                                    break;
                                case '2':        
                                    RJ2_SetHigh();
                                    break;
                                case '3':       
                                    RJ3_SetHigh();
                                    break;
                                case '4':        
                                    RJ4_SetHigh();
                                    break;
                                case '5':       
                                    RJ5_SetHigh();
                                    break;
                                 case '6':        
                                    RJ6_SetHigh();
                                    break;
                                case '7':       
                                    RJ7_SetHigh();
                                    break;      
                            }
                        }
                        else 
                        {
                            if(rxdataPort60[6] == 'o' && rxdataPort60[7] == 'f' && rxdataPort60[8] == 'f') 
                            {
                                switch(rxdataPort60[4])
                                {
                                case '0':        
                                    RJ0_SetLow();
                                    break;
                                case '1':       
                                    RJ1_SetLow();
                                    break;
                                case '2':        
                                    RJ2_SetLow();
                                    break;
                                case '3':       
                                    RJ3_SetLow();
                                    break;
                                case '4':        
                                    RJ4_SetLow();
                                    break;
                                case '5':       
                                    RJ5_SetLow();
                                    break;
                                 case '6':        
                                    RJ6_SetLow();
                                    break;
                                case '7':       
                                    RJ7_SetLow();
                                    break;      
                                }
                            }
                        }
                    }
                    // reuse the RX buffer
                    TCP_InsertRxBuffer(&port60TCB, rxdataPort60, sizeof(rxdataPort60));
                }

                if(t_client >= socketTimeout)
                { 
                    // send board status message only once at 2 seconds
                    socketTimeout = t_client + 2;
                    pot = ADC_GetConversion(8);

                    sprintf(txdataPort60,"Pot: %d\n LED's state: %d\n", pot, LATJ);
                    //send data back to the source
                    TCP_Send(&port60TCB, txdataPort60, strlen(txdataPort60));
                }
            }
            break;
        case SOCKET_CLOSING:
            TCP_SocketRemove(&port60TCB);
            break;
        default:
            break;
            // we should not end up here
    }
}

void TCP_Client_Initialize(unsigned long Server_IP_a, unsigned long Server_IP_b, unsigned long Server_IP_c, unsigned long Server_IP_d, unsigned int Port)
{
    //remoteSocket.addr.s_addr = MAKE_IPV4_ADDRESS(192, 168, 100, 101);
    //uint_Port = remoteSocket.port = 60;
    
    remoteSocket.addr.s_addr = MAKE_IPV4_ADDRESS(Server_IP_a, Server_IP_b, Server_IP_c, Server_IP_d);
    remoteSocket.port = Port;
}